#ifndef __LOOPDETECTION_H__
#define __LOOPDETECTION_H__

#include "BasicBlock.h"
#include "Instruction.h"
#include "Type.h"
#include "Unit.h"
#include <string>
#include <unordered_map>
#include <unordered_set>
#include <vector>

// https://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm

/**
 * @brief Tarjan算法找强连通分量
 * 1. 初始化索引值为0，创建一个空栈 S。
 * 2. 对于图 G 中的每个顶点 v，如果 v 的索引未定义（即 v 还未被访问过），则调用
 * strongconnect 函数。
 * 3. 在 strongconnect 函数中，首先设置 v
 * 的深度索引为当前最小的未使用索引，然后将索引值加 1。将 v 压入栈 S，并标记 v
 * 为已在栈中。
 * 4. 考虑 v 的所有后继节点。对于每个从 v 到 w 的边，如果 w 的索引未定义（即 w
 * 还未被访问过），则递 归调用 strongconnect 函数，并更新 v 的 lowlink 值为
 * v.lowlink 和 w.lowlink 的最小值。如果 w 已在栈中，那么更新 v 的 lowlink 值为
 * v.lowlink 和 w.index 的最小值。
 * 5. 如果 v 的 lowlink 值等于其索引值，那么 v
 * 是一个根节点，开始生成一个新的强连通分量。重复以下步骤， 直到 w 不等于
 * v：从栈 S 中弹出 w，标记 w 为不在栈中，并将 w 添加到当前的强连通分量中。
 * 6. 输出当前的强连通分量。
 *
 */

using blockSet = std::vector<BasicBlock *>;

class LoopDetection {
    Unit *unit;
    std::set<BasicBlock *> visitedbb;
    std::unordered_set<blockSet *> loops;
    std::unordered_map<BasicBlock *, BasicBlock *> coreBBs; // 核心Loop

    int idxcnt;

  public:
    LoopDetection(Unit *unit) : unit(unit) {}
    ~LoopDetection();
    bool isBaseLoop(blockSet *);
    auto begin() { return loops.begin(); }
    auto end() { return loops.end(); }
    void run();
    void run2();
    void clear();

    BasicBlock *getEndbb(BasicBlock *loopbb);
    BasicBlock *getOutbb(BasicBlock *condbb);
    BasicBlock *getCondbb(BasicBlock *loopbb);
    void findNestLoop(BasicBlock *start, CondBrInstruction *cond_start);
    void printloop(blockSet *loop);
};
void test(Unit *unit);

#endif //__LOOPDETECTION_H__

/*
void run() {
    detector = LoopDetection(unit);
    detector->run();
    areaCount = 0;
    for (auto it = detector->begin(); it != detector->end(); it++) {
        if (!detector->isBaseLoop(*it))
            continue;
        auto loop = *it;
        // 这样可以由内向外处理嵌套loop
        do {
            // 处理loop
            loop = detector->getParentLoop(loop);
        } while (loop);
    }
}
*/