#ifndef _SCCP_H
#define _SCCP_H
#include <stack>
#include <list>
#include <algorithm>
#include <unordered_map>
#include <vector>
#include <queue>
#include <set>
#include <map>
#include <memory>
#include "Unit.h"
#include "Instruction.h"
#include "Type.h"
#include "BasicBlock.h"
#include "SymbolTable.h"
using namespace std;

class InstructionVisitor;

static inline double get_const_value(ConstantSymbolEntry* value) {
    return value->getValue();
}

struct ValueStatus {
    enum Status { BOT = 0, CONST, TOP };
    Status status;
    ConstantSymbolEntry* value;
    // 初始化
    ValueStatus(){};
    ValueStatus(Status status_, ConstantSymbolEntry* value_):status(status_), value(value_){};
    // 这里需要设置好value的type类型
    [[nodiscard]] bool is_bot() const { return status == BOT; }// 格值不确定
    [[nodiscard]] bool is_const() const { return status == CONST; } // 格值为常量
    [[nodiscard]] bool is_top() const { return status == TOP; } // 格值未初始化
    // 格的交运算, 求两个格的最大下界, 即赋值较小的格状态
    void operator^=(ValueStatus &b) {
        if (b.status < status) {
            status = b.status;
            value = b.value;
        } else if (status == b.status && status == CONST) {
            double x,y;
            if(value->getType()->isInt()){
                x = (int)get_const_value(value);
            }else{
                x = (float)get_const_value(value);
            }
            if(b.value->getType()->isInt()){
                y = (int)get_const_value(b.value);
            }else{
                y = (float)get_const_value(b.value);
            }
            if (x != y) {
                status = BOT;
                value = nullptr;
            }
        }
    }
    // 格的不相等判定
    // 不是const的只需要判断是否state相等即可
    // 是const的还需要判断const值是否相等
    bool operator!=(ValueStatus &b) const {
        if (status != b.status)
            return true;
        if (status != CONST)
            return false;
        double x,y;
        if(value->getType()->isInt()){
            x = (int)get_const_value(value);
        }else{
            x = (float)get_const_value(value);
        }
        if(b.value->getType()->isInt()){
            y = (int)get_const_value(b.value);
        }else{
            y = (float)get_const_value(b.value);
        }
        return x != y;
    }
};

// 设置Operand和ValueState的对应关系
class ValueMap {
  public:
    void clear() { value_map.clear(); }
    ValueStatus get(Operand *key) {
        //递归基情况, 如果该key为常量, 那么就可以直接设置值
            // std::cout<<"this is key:"<<key->toStr()<<"\n";
        if(value_map.count(key) == 0 && key->isConst()){
            auto constant = dynamic_cast<ConstantSymbolEntry*>(key->getSymbolEntry());
            // std::cout<<"this is contant:"<<constant->toStr()<<"\n";
            set(key, ValueStatus(ValueStatus::CONST, constant));
        }else if(value_map.count(key) == 0 && !key->isConst()){
            set(key, ValueStatus(ValueStatus::BOT, nullptr));
        }
        return value_map[key];
    }
    void set(Operand *key, ValueStatus value) { 
        value_map[key] = value; 
        }

  private:
    std::map<Operand*, ValueStatus> value_map;
};

class SCCP{
  public:
    explicit SCCP(Unit* _unit) : unit(_unit){
        instruction_visitor = std::make_unique<InstructionVisitor>(*this);
        // instruction_visitor = new InstructionVisitor(*this);
    };
    void execute();
    ConstantSymbolEntry *constFold(Instruction *inst);

    std::set<std::pair<BasicBlock *, BasicBlock *>> &get_marked() {
        return marked;
    }
    ValueMap &get_value_map() { return value_map; }
    std::vector<std::pair<BasicBlock *, BasicBlock *>> &get_cfg_worklist() {
        return cfg_worklist;
    }
    std::vector<Instruction *> &get_ssa_worklist() { return ssa_worklist; }
    ValueStatus getMapped(Operand *key) { return value_map.get(key); }

  private:
    void sccp(Function *f);
    ConstantSymbolEntry* cmpConstFold(CmpInstruction *inst, ConstantSymbolEntry *v1, ConstantSymbolEntry *v2);
    ConstantSymbolEntry* binConstFold(BinaryInstruction *inst, ConstantSymbolEntry *v1, ConstantSymbolEntry *v2);
    // 单目运算指令
    ConstantSymbolEntry* constFold(Instruction *inst, ConstantSymbolEntry *v);
    void replaceConstant(Function *f);
    // 修改cond跳转块
    static void condBrToJmp(Instruction *inst, BasicBlock *jmp_bb,
                            BasicBlock *invalid_bb, BasicBlock *bb);

    ValueMap value_map;
    std::set<std::pair<BasicBlock *, BasicBlock *>> marked;
    std::vector<std::pair<BasicBlock *, BasicBlock *>> cfg_worklist;
    std::vector<Instruction *> ssa_worklist;
    // 智能指针
    std::unique_ptr<InstructionVisitor> instruction_visitor;
    // InstructionVisitor* instruction_visitor;
    Unit* unit;
    // 替换标志
    bool replaceflag = false;
};

class InstructionVisitor {
  public:
    explicit InstructionVisitor(SCCP &sccp_pass)
        : sccp(sccp_pass), value_map(sccp_pass.get_value_map()),
          cfg_worklist(sccp_pass.get_cfg_worklist()),
          ssa_worklist(sccp_pass.get_ssa_worklist()){}
    void visit(Instruction *inst);

  private:
    //遍历phi指令
    void visit_phi(PhiInstruction *inst);
    //遍历cond指令
    void visit_br(CondBrInstruction *inst);
    //常量展开指令
    void visit_foldable(Instruction *inst);

    SCCP &sccp;
    ValueMap &value_map;
    //cfg边
    std::vector<std::pair<BasicBlock *, BasicBlock *>> &cfg_worklist;
    std::vector<Instruction *> &ssa_worklist;

    Instruction* inst_;
    BasicBlock* bb;
    ValueStatus prev_status;
    //游走状态指针
    ValueStatus cur_status;
    bool flagBr = false;
};

#endif