#include "Hoisting.h"

#define DEBUG_SWITCH_hs 0
#if DEBUG_SWITCH_hs
#define print(str) std::cout << str << "\n";
#else
#define print(str) //
#endif

// https://web.archive.org/web/20170806083023/http://www.compileroptimizations.com/category/hoisting.htm

bool Hoisting::invariantFind(int insttype) {
    for (auto ins : invariant) {
        if (ins.second->getInstType() == insttype) {
            return true;
        }
    }
    return false;
}

bool Hoisting::invariantNestCheck() {
    std::set<Operand *> store_addrs;
    for (auto ins : invariant) {
        if (ins.second->isStore()) {
            store_addrs.insert(ins.second->getOp()[0]);
        }
    }
    for (auto ins : invariant) {
        if (ins.second->isLoad()) {
            if (store_addrs.find(ins.second->getOp()[1]) != store_addrs.end()) {
                return false;
            }
        }
    }
    return true;
}

bool Hoisting::rapidMove(blockSet *loop) {
    print("======== rapidMove ========");
    std::vector<Instruction *> geps;
    for (auto invar : invariant) {
        print(invar.second->getDef()->toStr());
        if (!(invar.second->isGep() &&
              invar.second->getDef()->getUse()[0]->isCall() &&
              !(invar.second->getUse()[0]->isConst() ||
                invar.second->getUse()[0]->isGlobal()) &&
              invar.second->getUse()[0]->getDef() &&
              invar.second->getUse()[0]->getDef()->isAlloc()))
            return false;
        else {
            geps.push_back(invar.second);
        }
    }
    BasicBlock *coreLoop = nullptr;
    for (auto bb : *loop) {
        if (bb->existsInPred(bb))
            coreLoop = bb;
        print(bb->getNo());
    }
    if (coreLoop == nullptr)
        return false;
    if (!((*loop->begin())->getNumOfPred() == 2 &&
          (*loop->begin())->getNumOfSucc() == 2))
        return false;
    auto loop_cond = dynamic_cast<CondBrInstruction *>(
        findinst(Instruction::COND, coreLoop));
    BasicBlock *entry = loop_cond->getFalseBranch();
    BasicBlock *restbb = nullptr;
    BasicBlock *start = (*loop->begin());
    auto phi =
        dynamic_cast<PhiInstruction *>(findinst(Instruction::PHI, start));
    for (auto pred : start->getPredBlock()) {
        if (phi->getSrcs()[pred] && !phi->getSrcs()[pred]->isConst()) {
            restbb = pred;
        }
    }
    assert(restbb);
    if (!findinst(Instruction::COND, restbb))
        return false;
    auto call =
        dynamic_cast<CallInstruction *>(findinst(Instruction::CALL, restbb));
    IdentifierSymbolEntry *funcSe =
        dynamic_cast<IdentifierSymbolEntry *>(call->getSymEntry());
    if (funcSe->isSysy() || funcSe->getName() == "llvm.memset.p0.i32")
        return false;
    std::vector<PhiInstruction *> phis;
    for (auto ins = restbb->begin(); ins != restbb->end();
         ins = ins->getNext()) {
        if (!ins->isBin())
            continue;
        for (auto use : ins->getUse()) {
            if (use && use->getDef() && use->getDef()->isPhi()) {
                phis.push_back(dynamic_cast<PhiInstruction *>(use->getDef()));
            }
        }
    }
    auto phi2 =
        dynamic_cast<PhiInstruction *>(findinst(Instruction::PHI, restbb));
    BasicBlock *tmp = nullptr;
    for (auto src : phi2->getSrcs()) {
        if (src.first != entry)
            tmp = src.first;
    }
    assert(tmp);
    phi2->getSrcs().erase(tmp);
    print("phis num = " << phis.size());
    for (auto p : phis) {
        p->getParent()->remove(p);
        restbb->insertFront(p);
        BasicBlock *t = nullptr;
        for (auto src : p->getSrcs()) {
            if (src.first != restbb)
                t = src.first;
        }
        assert(t);
        p->getSrcs()[entry] = p->getSrcs()[t];
        p->getSrcs().erase(t);
    }
    phis.clear();
    call->getParent()->remove(call);
    entry->insertFront(call);
    
    for (auto gep : geps) {
        gep->getParent()->remove(gep);
        entry->insertFront(gep);
    }
    geps.clear();

    // 移除start的前驱
    if (start->existsInPred(restbb))
        start->removePred(restbb);
    // 移除rest的后继
    if (restbb->existsInSucc(start))
        restbb->removeSucc(start);
    // 处理rest的前驱后继
    restbb->getPredBlock().clear();
    restbb->addPred(entry);
    restbb->addPred(restbb);
    restbb->addSucc(restbb);

    auto rest_cond =
        dynamic_cast<CondBrInstruction *>(findinst(Instruction::COND, restbb));
    rest_cond->setTrueBranch(restbb);

    std::cout << "rapid move done.\n";
    print("======== rapidMove end ========");
    return true;
}

void Hoisting::execute(bool aggressive) {
    detector = new LoopDetection(unit);
    detector->run();
    print("======== hoisting pass ========");
    for (auto it = detector->begin(); it != detector->end(); it++) {
        // if (!detector->isBaseLoop(*it))
        //     continue;
        // if ((*it)->size() > 1)
        //     continue;m
        auto loop = *it;
        FunctionType *funcType = dynamic_cast<FunctionType *>(
            (*loop->begin())->getParent()->getSymPtr()->getType());
        std::vector<Type *> paramlist = funcType->getParamsType();
        int arrParamCnt = 0;
        int intParamCnt = 0;
        int floatParamCnt = 0;
        for (auto param : paramlist) {
            if (param->isArray()) {
                arrParamCnt++;
            } else if (param->isFloat()) {
                floatParamCnt++;
            } else {
                intParamCnt++;
            }
        }
        // std::cout<<paramlist.size()<<" "<<intParamCnt<<"
        // "<<arrParamCnt<<"\n";
        if (paramlist.size() == 4 && intParamCnt == 1 && arrParamCnt == 3) {
            return;
        }
        // do {
        invariant.clear();
        findDef(loop);
        findInvariants(loop);
        if (invariant.size() > 0) {
            if (!invariantNestCheck())
                continue;
            moveInvariantsOut(loop);
        }
        // loop = detector->getParentLoop(loop);
        // } while (loop);
    }
    // this->aggressive = aggressive;
    if (aggressive) {
        detector = new LoopDetection(unit);
        detector->run2();
        print("======== hoisting pass2 ========");
        for (auto it = detector->begin(); it != detector->end(); it++) {
            auto loop = *it;
            if (loop->size() <= 1)
                continue;
            // while (true) {
            invariant.clear();
            findDef(loop);
            findInvariants(loop);
            if (invariant.size() > 0) {
                if (rapidMove(loop) == 1)
                    continue;
                if (invariantFind(Instruction::STORE) == 0)
                    continue;
                if (!invariantNestCheck())
                    continue;
                moveInvariantsOut(loop);
                findInvariantLoop(loop);
            }
            // else
            // break;
            // }
        }
    }
    print("======== hoisting end ========");
}

void Hoisting::findDef(blockSet *loop) {
    for (auto bb : *loop) {
        auto head = bb->end();
        SymbolEntry *se;
        for (auto instr = head->getNext(); instr != head;
             instr = instr->getNext()) {
            if (instr->isStore()) {
                if (instr->getUse()[0]->getDef() &&
                    instr->getUse()[0]->getDef()->getParent() != bb)
                    continue;
                // Fix: 这有问题啊.........
                se = instr->getOpse()[0];
                if (se == nullptr)
                    continue;
                print("def-store: " << se->toStr());
                defInLoop.insert(se);
                // std::cout<<"def-opd: "<<se->toStr()<<"\n";
            } else if (instr->isPhi() && instr->getDst()->usersNum() > 0) {
                se = instr->getOpse()[0];
                if (se == nullptr)
                    continue;
                print("def-phi: " << se->toStr());
                defInLoop.insert(se);
                // std::cout<<"def-phi-opd: "<<se->toStr()<<"\n";
            }
        }
    }
}

bool Hoisting::FunctionCheck(CallInstruction *call) {
    IdentifierSymbolEntry *funcSe =
        dynamic_cast<IdentifierSymbolEntry *>(call->getSymEntry());
    print("func: " << funcSe->getName());
    if (funcSe->isSysy() || funcSe->getName() == "llvm.memset.p0.i32")
        return false;
    Function *func = dynamic_cast<Function *>(funcSe->getFunction());
    for (auto bb : func->getBlockList()) {
        for (auto instr = bb->begin(); instr != bb->end();
             instr = instr->getNext()) {
            if (instr->isStore()) {
                auto src = instr->getDef();
                // std::cout << src->toStr() << "\n";
                if (src->isGlobal()) {
                    // WARN 这里能找到吗？
                    if (instr->getUse()[0]->isParam())
                        continue;
                    return false;
                }
                if (src->getDef()->isGep()) {
                    auto gep = dynamic_cast<GepInstruction *>(src->getDef());
                    if (dynamic_cast<IdentifierSymbolEntry *>(gep->getArrSe())
                            ->isParam())
                        print("FunctionCheck 0");
                    return false;
                }
            } else if (instr->isCall()) {
                if (dynamic_cast<CallInstruction *>(instr)->getSymEntry() ==
                    funcSe)
                    return false;
                if (!FunctionCheck((CallInstruction *)instr))
                    return false;
            }
        }
    }
    print("FunctionCheck 1");
    return true;
}

bool Hoisting::checkPhiControl(blockSet *loop, PhiInstruction *phi,
                               BasicBlock *outbb, BasicBlock *bb) {
    print("============= checkPhiControl =============");
    for (auto src : phi->getSrcs()) {
        if (src.first != bb && !src.second->isConst() &&
            !src.second->isGlobal()) {
            if (src.second->getDef()->isPhi()) {
                auto phi2 =
                    dynamic_cast<PhiInstruction *>(src.second->getDef());
                for (auto src2 : phi2->getSrcs()) {
                    if (src2.first == outbb)
                        continue;
                    else if (src2.second->isConst() &&
                             std::find(loop->begin(), loop->end(),
                                       src2.first) == loop->end()) {
                        entry = src2.first;
                        continue;
                    }
                    return false;
                }
            }
        }
    }
    if (entry && findinst(Instruction::UNCOND, entry) == nullptr)
        return false;
    return true;
}

void Hoisting::moveInvariantLoop(BasicBlock *bb, BasicBlock *outbb) {

    print("============= moveInvariantLoop =============");
    BasicBlock *bb_pred = nullptr;
    print(bb->getPrevbb().size());
    for (auto pred : bb->getPrevbb()) {
        print("pred = " << pred->getNo());
        if (pred != bb)
            bb_pred = pred;
    }
    assert(bb_pred);
    // BasicBlock *invarBB = new BasicBlock(bb->getParent());
    auto uncond_entry = dynamic_cast<UncondBrInstruction *>(
        findinst(Instruction::UNCOND, entry));
    auto uncond_pred = dynamic_cast<UncondBrInstruction *>(
        findinst(Instruction::UNCOND, bb_pred));
    auto entry_tobb = uncond_entry->getBranch();
    auto cond_bb =
        dynamic_cast<CondBrInstruction *>(findinst(Instruction::COND, bb));
    BinaryInstruction *add;
    print("bb_pred = " << bb_pred->getNo());
    if (findinst(Instruction::PHI, outbb)) {
        auto phi =
            dynamic_cast<PhiInstruction *>(findinst(Instruction::PHI, outbb));
        if (phi->getSrcs().size() > 1)
            return;
        auto cmp =
            dynamic_cast<CmpInstruction *>(findinst(Instruction::CMP, outbb));
        if (!cmp->getUse()[0]->getDef()->isBin())
            return;
        auto bin = cmp->getUse()[0]->getDef();
        if (!bin->getUse()[0]->getDef()->isPhi())
            return;
        auto phi_ = dynamic_cast<PhiInstruction *>(bin->getUse()[0]->getDef());
        assert(phi_->getSrcs()[entry]);
        phi_->getSrcs()[bb_pred] = phi_->getSrcs()[entry];
        phi_->getSrcs().erase(entry);
        bin->getParent()->remove(bin);
        outbb->insertAfter(bin, phi);
        phi_->getParent()->remove(phi_);
        outbb->insertAfter(phi_, phi);

        auto dst = new Operand(new TemporarySymbolEntry(
            TypeSystem::intType, SymbolTable::getLabel()));
        phi->getDef()->replaceAllUseWith(dst);
        assert(phi->getSrcs()[outbb]);
        add = new BinaryInstruction(BinaryInstruction::ADD, dst, phi->getDef(),
                                    phi->getSrcs()[outbb]);
        add->push_back_opse(phi->getIdentse());
        add->push_back_opse(phi->getSrcs()[outbb]->getSymbolEntry());
        auto zero =
            new Operand(new ConstantSymbolEntry(TypeSystem::intType, 0));
        outbb->insertAfter(add, phi_);
        phi->getSrcs()[bb_pred] = zero;
        // phi->getSrcs()[bb_pred] = phi->getSrcs()[outbb];
        // phi->getSrcs().erase(outbb);
        phi->getSrcs()[outbb] = dst->getUse()[0]->getDef();
        print("outbb = " << outbb->getNo());
        auto cond_outbb = dynamic_cast<CondBrInstruction *>(
            findinst(Instruction::COND, outbb));
        // print(cond_outbb->getDef()->toStr());
        // getchar();
        cond_outbb->getTrueBranch()->removePred(outbb);
        outbb->removeSucc(cond_outbb->getTrueBranch());
        cond_outbb->setTrueBranch(outbb);
        outbb->addPred(outbb);
        outbb->addSucc(outbb);

    } else
        return;
    print("start change");
    for (auto ins = bb->begin(); ins != bb->end(); ins = ins->getNext()) {
        if (ins->isPhi()) {
            auto phi = dynamic_cast<PhiInstruction *>(ins);
            print("phi = " << phi->getDef()->toStr());
            assert(phi->getSrcs()[bb_pred]);
            auto op = phi->getSrcs()[bb_pred];
            if (op->isConst() || op->isGlobal()) {
                phi->getSrcs().erase(bb_pred);
                phi->getSrcs()[entry] = op;
                continue;
            }
            auto accum_phi = dynamic_cast<PhiInstruction *>(op->getDef());
            assert(accum_phi->isPhi());
            auto accum_init = accum_phi->getSrcs()[entry];
            if (!accum_init->isConst() || dynamic_cast<ConstantSymbolEntry *>(
                                              accum_init->getSymbolEntry())
                                                  ->getValue() != 0)
                return;
            // auto op = phi->getSrcs()[bb_pred];
            phi->getSrcs().erase(bb_pred);
            phi->getSrcs()[entry] = accum_init;
        } else {
            break;
        }
    }
    for (auto ins = entry_tobb->begin(); ins != entry_tobb->end();
         ins = ins->getNext()) {
        if (ins->isPhi()) {
            auto phi = dynamic_cast<PhiInstruction *>(ins);
            auto op = phi->getSrcs()[entry];
            phi->getSrcs().erase(entry);
            phi->getSrcs()[bb] = op;
            // for (auto src : phi->getSrcs()) {
            //     if(src.first==entry){
            //         phi->getSrcs()[entry]=

            //     }
            // }
        } else {
            break;
        }
    }

    uncond_pred->setBranch(outbb);
    outbb->removePred(bb);
    bb->removeSucc(outbb);
    bb_pred->addSucc(outbb);
    outbb->addPred(bb_pred);

    bb->removePred(bb_pred);
    bb_pred->removeSucc(bb);

    entry_tobb->removePred(entry);
    entry->removeSucc(entry_tobb);
    entry_tobb->addPred(bb);
    bb->addSucc(entry_tobb);
    entry->addSucc(bb);
    bb->addPred(entry);

    cond_bb->setFalseBranch(uncond_entry->getBranch());
    uncond_entry->setBranch(bb);
}

void Hoisting::findInvariantLoop(blockSet *loop) {
    print("============= findInvariantLoop =============");
    for (auto bb : *loop) {
        if (findinst(Instruction::STORE, bb)) {
            return;
        }
    }
    print("No store");
    for (auto bb : *loop) {
        entry = nullptr;
        if (findinst(Instruction::COND, bb)) {
            auto cond = dynamic_cast<CondBrInstruction *>(
                findinst(Instruction::COND, bb));
            if (cond->getTrueBranch() != bb)
                continue;
            auto outbb = cond->getFalseBranch();
            for (auto instr = bb->begin(); instr != bb->end();
                 instr = instr->getNext()) {
                if (instr->isPhi()) {
                    auto phi = dynamic_cast<PhiInstruction *>(instr);
                    if (!checkPhiControl(loop, phi, outbb, bb))
                        return;
                }
            }
            print("bb = " << bb->getNo() << " outbb = " << outbb->getNo());
            moveInvariantLoop(bb, outbb);
            return;
        }
    }
}

// 寻找不变量
void Hoisting::findInvariants(blockSet *loop) {

    std::unordered_set<Instruction *> allInstructions;

    // 遍历循环中的所有基本块，把所有的指令加入到集合中
    for (auto bb : *loop) {
        auto head = bb->end();
        for (auto instr = head->getNext(); instr != head;
             instr = instr->getNext()) {
            for (auto opd : instr->getUse()) {
                print(opd->toStr() << " " << opd->isInvariant());
                if (opd->isConst() ||
                    aggressive && opd->getDef() && opd->getDef()->isAlloc()
                    // || opd->getSymbolEntry()->getType()->isArray()
                ) {
                    print("1");
                    opd->setInvariant(1);
                    // print("const: " << opd->toStr())
                } else if (opd->isGlobal() &&
                           defInLoop.find(opd->getSymbolEntry()) !=
                               defInLoop.end()) {
                    print("2");
                    continue;
                } else if (opd->getOpdSe() &&
                           defInLoop.find(opd->getOpdSe()) == defInLoop.end()) {
                    print("3");
                    if (opd->getOpdSe()->getType()->isArray()
                        // || dynamic_cast<IdentifierSymbolEntry
                        // *>(opd->getOpdSe())
                        //         ->isGlobal()
                    )
                        continue; // todo: hosting数组/~全局变量~

                    // std::cout << "opd se: " << opd->getOpdSe() << " "
                    //           << opd->getOpdSe()->toStr() << "\n ";
                    opd->setInvariant(1);
                } else {
                    opd->setInvariant(0);
                }
            }
            allInstructions.insert(instr);
        }
    }

    bool change = false;
    // 持续对循环中的所有基本块进行处理，直到没有新的循环不变量生成
    do {
        change = false;
        // 对每个基本块进行处理
        for (auto bb : *loop) {
            // 对每个基本块中的指令进行处理
            auto head = bb->end();
            for (auto instr = head->getNext(); instr != head;
                 instr = instr->getNext()) {
                // print("======= " << instr->getInstName() << " START
                // =======");

                // 跳过一些特定类型的指令以及在循环中没有定义的指令
                if (instr->isCmp() || instr->isUncond() || instr->isCond() ||
                    // instr->isCall() ||
                    instr->isRet() || instr->isAlloc() || instr->isFtos() ||
                    instr->isStof() || instr->isBitcast() ||
                    (!aggressive && instr->isGep()) || // todo: hosting数组
                    allInstructions.find(instr) == allInstructions.end()) {
                    continue;
                }
                // if (instr->getDef()&&instr->getDef()->usersNum() < 1) {
                //     bb->remove(instr);
                //     continue;
                // }
                // 检查该指令的所有操作数是否都是循环不变量
                bool invar = true;
                if (instr->isCall()) {
                    CallInstruction *call =
                        dynamic_cast<CallInstruction *>(instr);
                    invar = FunctionCheck(call);
                    std::vector<Operand *> paramlist = call->getParams();
                    for (auto param : paramlist) {
                        if (!param->isInvariant()) {
                            invar = false;
                            break;
                        }
                    }
                } else {
                    bool cmpSrc = 0;
                    // 全局变量binary常量的cond
                    for (auto ins : instr->getDef()->getUse()) {
                        if (ins->isCmp()) {
                            cmpSrc = 1;
                            break;
                        }
                    }
                    if (cmpSrc)
                        continue;

                    for (auto opd : instr->getUse()) {
                        // std::cout<<"op = "<<opd->toStr()<<"\n";
                        print(instr->getInstName()
                              << " invar check: " << opd->toStr()
                              << " bool: " << opd->isInvariant());
                        if (!opd->isInvariant()) {
                            invar = false;
                            break;
                        }
                    }
                    if (instr->isStore()) {
                        if (!instr->getDef()->isInvariant())
                            invar = false;
                    }
                }
                if (invar) {
                    allInstructions.erase(instr);
                    invariant.push_back(std::make_pair(bb, instr));
                    if (instr->isStore())
                        instr->getUse()[0]->setInvariant(1);
                    else {
                        instr->getDef()->setInvariant(1);
                    }
                    change = true;
                }
            }
        }
    } while (change);

    bool allLoad = 1;
    print("============ LoopInvariants =============");
    for (auto pair : invariant) {
        if (!pair.second->isLoad())
            allLoad = 0;
        print("in bb: " << pair.first->getNo()
                        << "\tVal: " << (pair.second)->getInstName());
    }
    if (allLoad)
        invariant.clear();
}

// 外提不变量
void Hoisting::moveInvariantsOut(blockSet *loop) {
    bool allphi = 1;
    for (auto pair : invariant) {
        if (!pair.second->isPhi())
            allphi = 0;
    }
    if (allphi) {
        invariant.clear();
        return;
    }

    BasicBlock *loopBB = *(loop->begin());
    BasicBlock *condBB = detector->getCondbb(loopBB);
    BasicBlock *endBB = detector->getEndbb(loopBB);

    BasicBlock *invarBB = new BasicBlock(condBB->getParent());
    print("== loopbb = " << loopBB->getNo() << " condbb = " << condBB->getNo()
                         << " endbb = " << endBB->getNo());
    print("  == new bb = " << invarBB->getNo());

    loopBB->removePred(condBB);
    loopBB->addPred(invarBB);
    condBB->removeSucc(loopBB);
    condBB->addSucc(invarBB);
    invarBB->addSucc(loopBB);
    invarBB->addPred(condBB);

    auto head_cond = condBB->end();
    for (auto instr = head_cond->getNext(); instr != head_cond;
         instr = instr->getNext()) {
        if (instr->isCond()) {
            dynamic_cast<CondBrInstruction *>(instr)->setTrueBranch(invarBB);
            break;
        } else if (instr->isUncond()) {
            dynamic_cast<UncondBrInstruction *>(instr)->setBranch(invarBB);
            break;
        }
    }
    for (auto pair : invariant) {
        Instruction *instr = pair.second;
        // print("remove: "<<instr->getInstName());
        pair.first->remove(instr);
        if (instr->isPhi()) {
            invarBB->insertFront(instr);
        } else
            invarBB->insertBack(instr);
    }

    new UncondBrInstruction(loopBB, invarBB);
    // new UncondBrInstruction(invarBB, condBB);

    // std::vector<BasicBlock *> labels;
    // std::vector<Operand *> vals;
    // std::vector<BasicBlock *> inloopLabels;
    // std::vector<Operand *> inloopVals;
    auto head = invarBB->end();
    // 处理phi指令
    for (auto instr = head->getNext(); instr != head;
         instr = instr->getNext()) {
        if (!instr->isPhi())
            continue;
        PhiInstruction *phi = dynamic_cast<PhiInstruction *>(instr);
        // labels.clear();
        // vals.clear();
        // inloopVals.clear();
        // inloopLabels.clear();
        // auto phiBBs = phi->getPhiBBs();
        // auto phiOperands = phi->getPhiOperands();

        auto bblist = invarBB->getParent()->getBlockList();
        std::set<BasicBlock *> removebbs;
        for (auto src : phi->getSrcs()) {
            // print("PHIBB " << src.first->getNo() << " " <<
            // src.second->toStr());
            if (std::find(bblist.begin(), bblist.end(), src.first) ==
                bblist.end()) {
                removebbs.insert(src.first);
            }
        }
        for (auto b : removebbs) {
            print("remove bb = " << b->getNo() << " "
                                 << phi->getSrc(b)->toStr());
            phi->getSrcs().erase(b);
        }
        removebbs.clear();

        // 就应该只有两个
        BasicBlock *inloop = nullptr;
        BasicBlock *outloop = nullptr;
        std::map<BasicBlock *, Operand *> in_loop_srcs;
        std::map<BasicBlock *, Operand *> out_loop_srcs;
        for (auto src : phi->getSrcs()) {
            if (std::find(loop->begin(), loop->end(),
                          dynamic_cast<BasicBlock *>(src.first)) !=
                loop->end()) {
                inloop = src.first;
            } else {
                outloop = src.first;
            }
        }
        if (inloop != nullptr && phi->getSrcs()[inloop]) {
            auto val = phi->getSrcs()[inloop];
            // print("invar src: " << outloop->getNo() << " " << val->toStr());
            phi->getSrcs()[outloop] = val;
            phi->getSrcs().erase(inloop);
        }
        // for (unsigned i = 0; i < 2; i++) {
        //     if (loop->find(dynamic_cast<BasicBlock *>(phiBBs[i])) !=
        //         loop->end()) {
        //         inloopLabels.push_back(phiBBs[i]);
        //         inloopVals.push_back(phiOperands[i]);
        //     } else {
        //         labels.push_back(phiBBs[i]);
        //         vals.push_back(phiOperands[i]);
        //         phiOperands[i]->removeUse(phi);
        //     }
        // }

        // if (labels.size() == 0)
        //     continue;
        // if (labels.size() == 1) {
        //     for (unsigned i = 0; i < instr->getNumOperand(); i += 2) {
        //         if (vals[0] != v[i])
        //             continue;
        //         instr->setOperand(i + 1, invarBB);
        //         labels[0]->removeUse(instr, i + 1);
        //     }
        //     continue;
        // }
        // else {

        //     for (int j = 0; j < 2; j++) {
        //         if (phi->getPhiBBs()[j] == labels[0]) {
        //             print(phi->getPhiBBs()[j]->getNo()
        //                   << " " << labels[0]->getNo());

        //             phi->getPhiOperands_v()[j] = inloopVals[0];

        //             phi->getSrcs()[labels[0]] = inloopVals[0];

        //             print(phi->getPhiOperands_v()[j]->toStr()
        //                   << " " << inloopVals[0]->toStr());

        //             phi->getPhiOperands_v()[j] = inloopVals[0];
        //         }
        //     }
        // }
        // auto new_phi = PhiInst::createPhi(v[0]->getType(), invarBB);
        // // assert(new_phi->getNumOperand() == 0);
        // for (unsigned i = 0; i < labels.size(); i++) {
        //     new_phi->addOperand(vals[i]);
        //     new_phi->addOperand(labels[i]);
        // }
        // instr->clearOps();
        // for (unsigned i = 0; i < inloopLabels.size(); i++) {
        //     instr->addOperand(inloopVals[i]);
        //     instr->addOperand(inloopLabels[i]);
        // }
        // instr->addOperand(new_phi);
        // instr->addOperand(invarBB);
        // invarBB->addInstrBegin(new_phi);
    }
    auto head_loop = loopBB->end();
    for (auto instr = head_loop->getNext(); instr != head_loop;
         instr = instr->getNext()) {
        if (!instr->isPhi())
            continue;
        PhiInstruction *phi = dynamic_cast<PhiInstruction *>(instr);
        if (phi->getSrcs().find(condBB) != phi->getSrcs().end()) {
            auto val = phi->getSrcs()[condBB];
            print("loop src: " << condBB->getNo() << " " << val->toStr());
            phi->getSrcs()[invarBB] = val;
            phi->getSrcs().erase(condBB);
        }
        // auto phiBBs = phi->getPhiBBs();
        // auto phiOperands = phi->getPhiOperands();

        // for (int j = 0; j < 2; j++) {
        //     if (phiBBs[j] != loopBB) {

        //         phi->getPhiBBs()[j] = invarBB;
        //         phi->getSrcs()[invarBB] = phi->getSrcs()[phiBBs[j]];
        //         phi->getSrcs().erase(phiBBs[j]);
        //     }
        // }
        // 删掉原有bb块
        // phi->getSrcs().erase(loopBB);
    }
    std::cout << "yes hoist done.\n";
}