%code top{
    #include <cstring>
    #include <stack>
    #include <vector>
    #include <iostream>
    #include <assert.h>
    #include <algorithm>
    #include "parser.h"
    extern int yylineno;
    extern char* yytext;
    extern Ast ast;
    extern bool *fcheck;

    #define DEBUG_SWITCH_TYPE_CHECK 1
    #if DEBUG_SWITCH_TYPE_CHECK
    #define printTyCk(str) std::cerr<<"[error]:"<<str<<"\n";
    #else
    #define printTyCk(str) //
    #endif

    int yylex();
    int yyerror(char const*);
    Type* CurType;
    Type* ReturnType;
    bool isConst;
    ArrayType* curArrayType;
    int idx;
    double* arrayValue;
    std::vector<InitValueListExpr*> InitValuesVec;
    std::vector<StmtNode*> WhileNodesVec;
    InitValueListExpr* top;
    int leftCnt = 0;
    int WhileIndex = 0;
    int AllParamNum = 0;
    int IntParamNum = 0;
    int FloatParamNum = 0;
    int NoZeroNum = 0;
}

%code requires {
    #include "SymbolTable.h"
    #include "Ast.h"
    #include "Type.h"
}

%union {
    char* StrType;
    int IntType;
    float FloatType;
    StmtNode* StateType;
    ExprNode* ExprType;
    Type* type;
    SymbolEntry* se;
}

%start Program
%token <IntType> DECIMIAL OCTAL HEXADECIMAL
%token <FloatType> DECIMAL_FLOAT HEXADECIMAL_FLOAT
%token <StrType> ID SYSY_FUNC_NOPARAM SYSY_FUNC_WITHPARAM
%token INT VOID FLOAT
%token IF ELSE WHILE
%token CONST
%token OR AND 
%token ADD SUB MUL DIV MOD
%token LESSEQUAL GREATER GREATEREQUAL LESS EQUAL NOTEQUAL NOT ASSIGN
%token COMMA SEMICOLON 
%token LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET 
%token RETURN CONTINUE BREAK

%type<StateType> States BlankState State 
%type<StateType> AssignState IfState WhileState ExprState BlockState BreakState ContinueState ReturnState
%type<StateType> DeclState ConstVarDeclState VarDecl VarDeclList VarDeclState
%type<StateType> FuncDecl FuncFParam FuncFParams FuncFParamsList
%type<ExprType> LVal
%type<ExprType> Cond PrimaryExpr UnaryExpr MulExpr AddExpr RelExpr EqExpr LAndExpr LOrExpr
%type<ExprType> InitArrList ArrayIndices InitArr
%type<ExprType> FuncRParams
%type<type> Type

%precedence THEN
%precedence ELSE
%%

Program
    : States {
        ast.setRoot($1);
    }
    ;
AssignState
    : LVal ASSIGN Cond SEMICOLON {
        $$ = new AssignStmt($1, $3);
    }
    ;
ExprState
    : Cond SEMICOLON {
        $$ = new ExprStmt($1);
    }
    ;
BlankState
    : SEMICOLON {
        $$ = new BlankStmt();
    }
    ;
BlockState
    : LBRACE {
        identifiers = new SymbolTable(identifiers);
    } 
      States RBRACE {
        $$ = new CompoundStmt($3);
        SymbolTable* ScopeTable = identifiers;
        identifiers = identifiers->getPrev();
        delete ScopeTable;
    }
    | LBRACE RBRACE {
        $$ = new CompoundStmt();
    }
    ;
IfState
    : IF LPAREN Cond RPAREN State %prec THEN {
        $$ = new IfStmt($3, $5);
    }
    | IF LPAREN Cond RPAREN State ELSE State {
        $$ = new IfElseStmt($3, $5, $7);
    }
    ;
WhileState
    : WHILE LPAREN Cond RPAREN {
        WhileIndex++;
        WhileStmt *whileNode = new WhileStmt($3);
        $<StateType>$ = whileNode;
        WhileNodesVec.push_back(whileNode);
    }
    State {
        WhileStmt *whileNode = dynamic_cast<WhileStmt*>($<StateType>5); 
        whileNode->setStmt($6);
        $$=whileNode;
        WhileNodesVec.pop_back();
        WhileIndex--;
    }
    ;
BreakState
    : BREAK SEMICOLON {
        if(WhileIndex <= 0){
            printTyCk("Continue is not in while statement.");
            assert(0);
        }
        $$ = new BreakStmt(WhileNodesVec.back());
    }
    ;
ContinueState
    : CONTINUE SEMICOLON {
        if(WhileIndex <= 0){
            printTyCk("Break is not in while statement.")
            assert(0);
        }
        $$ = new ContinueStmt(WhileNodesVec.back());
    }
    ;
ReturnState
    : RETURN SEMICOLON {
        $$ = new ReturnStmt();
    }
    | RETURN Cond SEMICOLON {
        if (($2->getType()->isFloat() && ReturnType->isInt()) ||
            ($2->getType()->isInt() && ReturnType->isFloat())) {
            $$ = new ReturnStmt(new ImplicitCastExpr($2, ReturnType));
        } else {
            $$ = new ReturnStmt($2);            
        }
    }
    ;
State
    : AssignState {$$=$1;}
    | BlankState {$$ = $1;}
    | ExprState {$$ = $1;}
    | BlockState {$$=$1;}
    | IfState {$$ = $1;}
    | WhileState {$$ = $1;}
    | ContinueState {$$=$1;}
    | BreakState {$$=$1;}
    | FuncDecl {$$ = $1;}
    | DeclState {$$ = $1;}
    | ReturnState {$$ = $1;}
    ;
States
    : State {$$=$1;}
    | States State{
        $$ = new SeqNode($1, $2);
    }
    ;
LVal
    : ID {
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr){
            printTyCk("identifier " << (char*)$1 << " is not defined.");
        }
        $$ = new Id(se);
        delete []$1;
    }
    | SYSY_FUNC_NOPARAM {
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr){
            printTyCk("identifier " << (char*)$1 << " is not defined.");
        }
        $$ = new Id(se);
        delete []$1;
    } 
    | SYSY_FUNC_WITHPARAM {
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr){
            printTyCk("identifier " << (char*)$1 << " is not defined.");
        }
        $$ = new Id(se);
        delete []$1;
    }
    | ID ArrayIndices{
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr){
            printTyCk("identifier "<< (char*)$1 << " is not defined.");
        }
        $$ = new Id(se, $2);
        delete []$1;
    }
    ; 
PrimaryExpr
    : LPAREN Cond RPAREN {
        $$ = $2;
    }
    | LVal {
        IdentifierSymbolEntry *id = dynamic_cast<IdentifierSymbolEntry*>($1->getSymbolEntry());
        if(id->getConst()){
            if(id->getType()->isInt() || id->getType()->isFloat()){
                SymbolEntry *cs = new ConstantSymbolEntry(id->getType(), id->getValue());
                $$ = new Constant(cs);
            }
            else{
                $$ = $1;
            }
        }else{
            $$ = $1;
        }
    }
    | DECIMIAL {
        ConstantSymbolEntry *se = new ConstantSymbolEntry(TypeSystem::intType, $1);
        $$ = new Constant(se);
    }
    | OCTAL {
        ConstantSymbolEntry *se = new ConstantSymbolEntry(TypeSystem::intType, $1);
        $$ = new Constant(se);
    }
    | HEXADECIMAL {
        ConstantSymbolEntry *se = new ConstantSymbolEntry(TypeSystem::intType, $1);
        $$ = new Constant(se);
    }
    | DECIMAL_FLOAT {
        *fcheck=1;
        ConstantSymbolEntry *se = new ConstantSymbolEntry(TypeSystem::floatType, $1);
        $$ = new Constant(se);
    }
    | HEXADECIMAL_FLOAT {
        *fcheck=1;
        ConstantSymbolEntry *se = new ConstantSymbolEntry(TypeSystem::floatType, $1);
        $$ = new Constant(se);
    }
    ;

UnaryExpr 
    : PrimaryExpr {$$ = $1;}
    | SYSY_FUNC_NOPARAM LPAREN RPAREN {
        globals->installSysFunc($1);
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr) {
            printTyCk("Fun: "  << (char*)$1 <<" is not defined.");
        } 
        $$ = new CallExpr(se);
    }
    | SYSY_FUNC_WITHPARAM LPAREN FuncRParams RPAREN {
        globals->installSysFunc($1);
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr) {
            printTyCk("Fun: "  << (char*)$1 <<" is not defined.");
        }
        $$ = new CallExpr(se, $3);
    }
    | ID LPAREN FuncRParams RPAREN {
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr) {
            printTyCk("Fun: "  << (char*)$1 <<" is not defined.");
        }
        $$ = new CallExpr(se, $3);
    }
    | ID LPAREN RPAREN {
        SymbolEntry* se = identifiers->lookup($1);
        if(se == nullptr){
            printTyCk("Fun: " << (char*)$1 <<" is not defined.");
        }
        $$ = new CallExpr(se);
    }
    | ADD UnaryExpr {
        $$ = $2;
    }
    | SUB UnaryExpr {
        Type* exprType = $2->getType();
        if($2->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *se = dynamic_cast<ConstantSymbolEntry*>($2->getSymbolEntry());
            ConstantSymbolEntry *cs = new ConstantSymbolEntry(exprType,-se->getValue());
            $$ = new UnaryExpr(cs, UnaryExpr::SUB, $2);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(exprType, SymbolTable::getLabel());
            $$ = new UnaryExpr(se, UnaryExpr::SUB, $2);
        }
    }
    | NOT UnaryExpr {
        if($2->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *se = dynamic_cast<ConstantSymbolEntry*>($2->getSymbolEntry());
            ConstantSymbolEntry *cs = new ConstantSymbolEntry(TypeSystem::boolType,!se->getValue());
            $$ = new UnaryExpr(cs, UnaryExpr::NOT, $2);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new UnaryExpr(se, UnaryExpr::NOT, $2);
        }
    }
    ;
MulExpr
    : UnaryExpr {$$ = $1;}
    | MulExpr MUL UnaryExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            if(ts1->getType()->isFloat() || ts2->getType()->isFloat()){
                cs = new ConstantSymbolEntry(TypeSystem::floatType,ts1->getValue()*ts2->getValue());
            }
            else{
                cs = new ConstantSymbolEntry(TypeSystem::intType,(int)ts1->getValue()*(int)ts2->getValue());
            }
            $$ = new BinaryExpr(cs, BinaryExpr::MUL, $1, $3);
        } else{
            if($1->getSymbolEntry()->isConstant()
                && dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry())->getValue() == 0){
                ConstantSymbolEntry *cs = new ConstantSymbolEntry($3->getType(), 0);
                $$ = new BinaryExpr(cs, BinaryExpr::MUL, $1, $3);
            } 
            else if($3->getSymbolEntry()->isConstant() 
                && dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry())->getValue() == 0){
                ConstantSymbolEntry *cs = new ConstantSymbolEntry($1->getType(), 0);
                $$ = new BinaryExpr(cs, BinaryExpr::MUL, $1, $3);
            } 
            else{
                SymbolEntry* se;
                if ($1->getType()->isFloat() || $3->getType()->isFloat()) {
                    se = new TemporarySymbolEntry(TypeSystem::floatType, SymbolTable::getLabel());
                    $$ = new BinaryExpr(se, BinaryExpr::MUL, $1, $3);
                } else {
                    se = new TemporarySymbolEntry(TypeSystem::intType, SymbolTable::getLabel());
                    $$ = new BinaryExpr(se, BinaryExpr::MUL, $1, $3);
                }
            }
        }
    }
    | MulExpr DIV UnaryExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            if(ts1->getType()->isFloat() || ts2->getType()->isFloat()){
                cs = new ConstantSymbolEntry(TypeSystem::floatType,ts1->getValue()/ts2->getValue());
            }
            else{
                cs = new ConstantSymbolEntry(TypeSystem::intType,(int)ts1->getValue()/(int)ts2->getValue());
            }
            $$ = new BinaryExpr(cs, BinaryExpr::DIV, $1, $3);
        }else{
            if($1->getSymbolEntry()->isConstant()
                && dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry())->getValue() == 0){
                ConstantSymbolEntry *cs = new ConstantSymbolEntry($3->getType(), 0);
                $$ = new BinaryExpr(cs, BinaryExpr::DIV, $1, $3);
            } 
            else{
                SymbolEntry* se;
                if ($1->getType()->isFloat() || $3->getType()->isFloat()) {
                    se = new TemporarySymbolEntry(TypeSystem::floatType, SymbolTable::getLabel());
                    $$ = new BinaryExpr(se, BinaryExpr::DIV, $1, $3);
                } else {
                    se = new TemporarySymbolEntry(TypeSystem::intType, SymbolTable::getLabel());
                    $$ = new BinaryExpr(se, BinaryExpr::DIV, $1, $3);
                }
            }
        }
    }
    | MulExpr MOD UnaryExpr {
        if ($1->getType()->isFloat() || $3->getType()->isFloat()) {
            printTyCk("Wrong! Mod needs integer operands!");
        }
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs =new ConstantSymbolEntry(TypeSystem::intType,(int)ts1->getValue() % (int)ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::MOD, $1, $3);
        }else{
            if($1->getSymbolEntry()->isConstant()
                && dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry())->getValue() == 0){
                ConstantSymbolEntry *cs = new ConstantSymbolEntry($3->getType(), 0);
                $$ = new BinaryExpr(cs, BinaryExpr::MOD, $1, $3);
            } 
            else{
                SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::intType, SymbolTable::getLabel());
                $$ = new BinaryExpr(se, BinaryExpr::MOD, $1, $3);
            }
        }
    }
    ;
AddExpr
    : MulExpr {$$ = $1;}
    | AddExpr ADD MulExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            if(ts1->getType()->isFloat() || ts2->getType()->isFloat()){
                cs = new ConstantSymbolEntry(TypeSystem::floatType,ts1->getValue() + ts2->getValue());
            }
            else{
                cs = new ConstantSymbolEntry(TypeSystem::intType,(int)ts1->getValue() + (int)ts2->getValue());
            }
            $$ = new BinaryExpr(cs, BinaryExpr::ADD, $1, $3);
        }else{
            SymbolEntry* se;
            if ($1->getType()->isFloat() || $3->getType()->isFloat()) {
                se = new TemporarySymbolEntry(TypeSystem::floatType, SymbolTable::getLabel());
                $$ = new BinaryExpr(se, BinaryExpr::ADD, $1, $3);
            } else {
                se = new TemporarySymbolEntry(TypeSystem::intType, SymbolTable::getLabel());
                $$ = new BinaryExpr(se, BinaryExpr::ADD, $1, $3);
            }
        }
    }
    | AddExpr SUB MulExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            if(ts1->getType()->isFloat() || ts2->getType()->isFloat()){
                cs = new ConstantSymbolEntry(TypeSystem::floatType,ts1->getValue() - ts2->getValue());
            }
            else{
                cs = new ConstantSymbolEntry(TypeSystem::intType,(int)ts1->getValue() - (int)ts2->getValue());
            }
            $$ = new BinaryExpr(cs, BinaryExpr::SUB, $1, $3);
        }else{
            SymbolEntry* se;
            if ($1->getType()->isFloat() || $3->getType()->isFloat()) {
                se = new TemporarySymbolEntry(TypeSystem::floatType, SymbolTable::getLabel());
                $$ = new BinaryExpr(se, BinaryExpr::SUB, $1, $3);
            } else {
                se = new TemporarySymbolEntry(TypeSystem::intType, SymbolTable::getLabel());
                $$ = new BinaryExpr(se, BinaryExpr::SUB, $1, $3);
            }
        }
    }
    ;
RelExpr
    : AddExpr {
        $$ = $1;
    }
    | RelExpr LESS AddExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() < ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::LESS, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::LESS, $1, $3);
        }
    }
    | RelExpr LESSEQUAL AddExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() <= ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::LESSEQUAL, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::LESSEQUAL, $1, $3);
        }
    }
    | RelExpr GREATER AddExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() > ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::GREATER, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::GREATER, $1, $3);
        }
    }
    | RelExpr GREATEREQUAL AddExpr {
        if($1->getSymbolEntry()->isConstant()&&$3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() >= ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::GREATEREQUAL, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::GREATEREQUAL, $1, $3);
        }
    }
    ;
EqExpr
    : RelExpr {$$ = $1;}
    | EqExpr EQUAL RelExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() == ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::EQUAL, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::EQUAL, $1, $3);
        }
    }
    | EqExpr NOTEQUAL RelExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() != ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::NOTEQUAL, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::NOTEQUAL, $1, $3);
        }
    }
    ;
LAndExpr
    : EqExpr {$$ = $1;}
    | LAndExpr AND EqExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() && ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::AND, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::AND, $1, $3);
        }
    }
    ;
LOrExpr
    : LAndExpr {$$ = $1;}
    | LOrExpr OR LAndExpr {
        if($1->getSymbolEntry()->isConstant() && $3->getSymbolEntry()->isConstant()){
            ConstantSymbolEntry *ts1 = dynamic_cast<ConstantSymbolEntry*>($1->getSymbolEntry());
            ConstantSymbolEntry *ts2 = dynamic_cast<ConstantSymbolEntry*>($3->getSymbolEntry());
            ConstantSymbolEntry *cs;
            cs = new ConstantSymbolEntry(TypeSystem::boolType,ts1->getValue() || ts2->getValue());
            $$ = new BinaryExpr(cs, BinaryExpr::OR, $1, $3);
        }else{
            SymbolEntry* se = new TemporarySymbolEntry(TypeSystem::boolType, SymbolTable::getLabel());
            $$ = new BinaryExpr(se, BinaryExpr::OR, $1, $3);
        }
    }
    ;
Cond
    : LOrExpr {$$ = $1;}
    ;

Type
    : INT {
        $$ = TypeSystem::intType;
        CurType = TypeSystem::intType;
    }
    | VOID {
        $$ = TypeSystem::voidType;
    }
    | FLOAT {
        *fcheck=1;
        $$ = TypeSystem::floatType;
        CurType = TypeSystem::floatType;
    }
    ;

DeclState
    : VarDeclState {$$ = $1;}
    | ConstVarDeclState {$$ = $1;}
    ;

VarDeclState
    : Type VarDeclList SEMICOLON {
        if ($1->isFloat()) {
            *fcheck=1;
        }
        $$ = $2;
    }
    ;

VarDeclList
    : VarDeclList COMMA VarDecl {
        $$ = $1;
        $1->setNext($3);
    } 
    | VarDecl {$$ = $1;}
    ;

VarDecl
    : ID {
        SymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(!identifiers->install($1, se)) {
            printTyCk("identifier " << (char*)$1 <<" is already defined.");
        }
        $$ = new DeclStmt(new Id(se));
        delete []$1;
    }
    | SYSY_FUNC_NOPARAM {
        SymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(!identifiers->install($1, se)) {
            printTyCk("identifier " << (char*)$1 <<" is already defined.");
        }
        $$ = new DeclStmt(new Id(se));
        delete []$1;
    } 
    | SYSY_FUNC_WITHPARAM {
        SymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(!identifiers->install($1, se)) {
            printTyCk("identifier " << (char*)$1 <<" is already defined.");
        }
        $$ = new DeclStmt(new Id(se));
        delete []$1;
    }
    | ID ASSIGN InitArrList {
        IdentifierSymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(isConst){
            se->setConst();
        }
        if(!identifiers->install($1, se)) {
            printTyCk("Identifier " << (char*)$1 << " is already defined.");
        }
        double val = $3->getValue();
        if (CurType->isInt() && $3->getType()->isFloat()) {
            // float temp = (float)val;
            // int temp1 = (int)temp;
            // val = (double)temp1;
            val = (int)val;
        }
        se->setValue(val);
        $$ = new DeclStmt(new Id(se), $3);
        delete []$1;
    }
    | SYSY_FUNC_NOPARAM ASSIGN InitArrList {
        IdentifierSymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(isConst){
            se->setConst();
        }
        if(!identifiers->install($1, se)) {
            printTyCk("Identifier " << (char*)$1 << " is already defined.");
        }
        double val = $3->getValue();
        if (CurType->isInt() && $3->getType()->isFloat()) {
            // float temp = (float)val;
            // int temp1 = (int)temp;
            // val = (double)temp1;
            val = (int)val;
        }
        se->setValue(val);
        $$ = new DeclStmt(new Id(se), $3);
        delete []$1;
    }
    | SYSY_FUNC_WITHPARAM ASSIGN InitArrList {
        IdentifierSymbolEntry* se = new IdentifierSymbolEntry(CurType, $1, identifiers->getLevel());
        if(isConst){
            se->setConst();
        }
        if(!identifiers->install($1, se)) {
            printTyCk("Identifier " << (char*)$1 << " is already defined.");
        }
        double val = $3->getValue();
        if (CurType->isInt() && $3->getType()->isFloat()) {
            // float temp = (float)val;
            // int temp1 = (int)temp;
            // val = (double)temp1;
            val = (int)val;
        }
        se->setValue(val);
        $$ = new DeclStmt(new Id(se), $3);
        delete []$1;
    }
    | ID ArrayIndices {
        std::vector<int> arrayDims;
        ExprNode* dimNode = $2;
        while(dimNode){
            arrayDims.push_back(dimNode->getValue());
            dimNode = dynamic_cast<ExprNode*>(dimNode->getNext());
        }
        Type *type = CurType;
        Type* topType;
        while(!arrayDims.empty()){
            topType = new ArrayType(type, arrayDims.back());
            if(type->isArray())
                (dynamic_cast<ArrayType*>(type))->setArrayType(topType);
            type = topType;
            arrayDims.pop_back();
        }
        curArrayType = dynamic_cast<ArrayType*>(type);
        IdentifierSymbolEntry* se = new IdentifierSymbolEntry(type, $1, identifiers->getLevel());
        if(!identifiers->install($1, se)) {
            printTyCk("Identifier " << (char*)$1 << " is already defined.");
        }
        se->setAllZero();
        $$ = new DeclStmt(new Id(se));
        delete []$1;
    }
    | ID ArrayIndices ASSIGN {
        std::vector<int> arrayDims;
        ExprNode* dimNode = $2;
        while(dimNode){
            arrayDims.push_back(dimNode->getValue());
            dimNode = dynamic_cast<ExprNode*>(dimNode->getNext());
        }
        Type* type = CurType;
        Type* topType;
        for(auto it = arrayDims.rbegin(); it != arrayDims.rend(); it++) {
            topType = new ArrayType(type, *it);
            if(type->isArray())
                (dynamic_cast<ArrayType*>(type))->setArrayType(topType);
            type = topType;
        }
        curArrayType = dynamic_cast<ArrayType*>(type);
        idx = 0;
        InitValuesVec.clear();
        SymbolEntry* se = new IdentifierSymbolEntry(type, $1, identifiers->getLevel());
        if(isConst)
            (dynamic_cast<IdentifierSymbolEntry*>(se))->setConst();
        $<se>$ = se;
        arrayValue = new double[curArrayType->getSize()];
        NoZeroNum = 0;
    }
      InitArrList {
        IdentifierSymbolEntry* identSe = dynamic_cast<IdentifierSymbolEntry*>($<se>4);
        identSe->setArrayValue(arrayValue);
        identSe->setNotZeroNum(NoZeroNum);
        if ((NoZeroNum == 0) || (dynamic_cast<InitValueListExpr*>($5))->isEmpty()){
            identSe->setAllZero();
            (dynamic_cast<InitValueListExpr*>($5))->setAllZero();
        }
        if (!identifiers->install($1, identSe)) {
            printTyCk("Identifier " << (char*)$1 << " is already defined.");
        }
        $$ = new DeclStmt(new Id($<se>4), $5);
        delete []$1;
    }
    ;

ConstVarDeclState
    : CONST Type VarDeclList SEMICOLON {
        if ($2->isFloat()) {
            *fcheck=1;
        }
        isConst = true;
        $$ = $3;
        isConst = false;
    }
    ;

ArrayIndices
    : 
    LBRACKET RBRACKET {
        $$ = new ExprNode(nullptr);
    }
    | LBRACKET Cond RBRACKET {
        $$ = $2;
    }
    | ArrayIndices LBRACKET Cond RBRACKET {
        $$ = $1;
        $1->setNext($3);
    }
    ;
InitArrList 
    : Cond {
        if (!$1->getType()->isInt() && !$1->getType()->isFloat()) {
            printTyCk("rval is invalid.");
        }
        $$ = $1;
        if (!InitValuesVec.empty()) {
            double val = $1->getValue();
            if (val)
                NoZeroNum++;
            if (CurType->isInt() && $1->getType()->isFloat()) {
                // float temp = (float)val;
                // int temp1 = (int)temp;
                // arrayValue[idx++] = (double)temp1;
                arrayValue[idx++] = (int)val;
            } else {
                arrayValue[idx++] = val;
            }

            Type* topType = InitValuesVec.back()->getSymbolEntry()->getType();
            if (topType == TypeSystem::intType || topType == TypeSystem::floatType) {
                if ((topType->isInt() && $1->getType()->isFloat()) ||
                    (topType->isFloat() && $1->getType()->isInt())) {
                    ImplicitCastExpr* temp = new ImplicitCastExpr($1, CurType);
                    InitValuesVec.back()->addExpr(temp);
                } else {
                    InitValuesVec.back()->addExpr($1);
                }
            } else {
                while (topType) {
                    if (((dynamic_cast<ArrayType*>(topType)))->getElementType() != TypeSystem::intType &&
                        ((dynamic_cast<ArrayType*>(topType)))->getElementType() != TypeSystem::floatType) {
                        topType = ((dynamic_cast<ArrayType*>(topType)))->getElementType();
                        SymbolEntry* se = new ConstantSymbolEntry(topType);
                        InitValueListExpr* list = new InitValueListExpr(se);
                        InitValuesVec.back()->addExpr(list);
                        InitValuesVec.push_back(list);
                    } else {
                        Type* elemType = ((dynamic_cast<ArrayType*>(topType)))->getElementType();
                        if ((elemType->isInt() && $1->getType()->isFloat()) ||
                            (elemType->isFloat() && $1->getType()->isInt())) {
                            ImplicitCastExpr* temp = new ImplicitCastExpr($1, elemType);
                            InitValuesVec.back()->addExpr(temp);
                        } else {
                            InitValuesVec.back()->addExpr($1);
                        }
                        while (InitValuesVec.back()->isFull() && InitValuesVec.size() != (long unsigned int)leftCnt) {
                            topType = ((ArrayType*)topType)->getArrayType();
                            InitValuesVec.pop_back();
                        }
                        break;
                    }
                }
            }
        }         
    }
    | LBRACE RBRACE {
        SymbolEntry* se;
        ExprNode* list;
        if(InitValuesVec.empty()){
            memset(arrayValue, 0, curArrayType->getSize());
            idx += curArrayType->getSize() / CurType->getSize();
            se = new ConstantSymbolEntry(curArrayType);
            list = new InitValueListExpr(se);
        }else{
            Type* type = (dynamic_cast<ArrayType*>(InitValuesVec.back()->getSymbolEntry()->getType()))->getElementType();
            int len = type->getSize() / CurType->getSize();
            memset(arrayValue + idx, 0, type->getSize());
            idx += len;
            se = new ConstantSymbolEntry(type);
            list = new InitValueListExpr(se);
            InitValuesVec.back()->addExpr(list);
            while(InitValuesVec.back()->isFull() && InitValuesVec.size() != (long unsigned int)leftCnt){
                InitValuesVec.pop_back();
            }
        }
        $$ = list;
    }
    | LBRACE {
        SymbolEntry* se;
        if(!InitValuesVec.empty())
            curArrayType = dynamic_cast<ArrayType*>((dynamic_cast<ArrayType*>(InitValuesVec.back()->getSymbolEntry()->getType()))->getElementType());
        se = new ConstantSymbolEntry(curArrayType);
        if (curArrayType->getElementType() != TypeSystem::intType &&
            curArrayType->getElementType() != TypeSystem::floatType) {
            curArrayType = dynamic_cast<ArrayType*>(curArrayType->getElementType());
        }
        InitValueListExpr* expr = new InitValueListExpr(se);
        if(!InitValuesVec.empty())
            InitValuesVec.back()->addExpr(expr);
        InitValuesVec.push_back(expr);
        $<ExprType>$ = expr;
        leftCnt++;
    } 
      InitArr RBRACE {
        leftCnt--;
        while (InitValuesVec.back() != $<ExprType>2 && InitValuesVec.size() > (long unsigned int)(leftCnt + 1))
            InitValuesVec.pop_back();
        if (InitValuesVec.back() == $<ExprType>2)
            InitValuesVec.pop_back();
        $$ = $<ExprType>2;
        if (!InitValuesVec.empty())
            while (InitValuesVec.back()->isFull() && InitValuesVec.size() != (long unsigned int)leftCnt) {
                InitValuesVec.pop_back();
            }
        int size = (dynamic_cast<ArrayType*>($$->getSymbolEntry()->getType()))->getSize() / CurType->getSize();
        while (idx % size != 0)
            arrayValue[idx++] = 0;
        if (!InitValuesVec.empty())
            curArrayType = dynamic_cast<ArrayType*>(
                (dynamic_cast<ArrayType*>(InitValuesVec.back()->getSymbolEntry()->getType()))->getElementType());
    }
    ;

InitArr
    : InitArrList {
        $$ = $1;
    }
    | InitArr COMMA InitArrList {
        $$ = $1;
    }
    ;

FuncDecl
    :
    Type ID {
        identifiers = new SymbolTable(identifiers);
        if ($1->isFloat()) *fcheck=1;
        IntParamNum = 0;
        FloatParamNum = 0;
        AllParamNum = 0;
        ReturnType = $1;
    }
    LPAREN FuncFParamsList RPAREN {
        Type* funcType;
        std::vector<Type*> vec;
        std::vector<SymbolEntry*> vec1;
        DeclStmt* temp = dynamic_cast<DeclStmt*>($5);
        while (temp) {
            vec.push_back(temp->getId()->getSymbolEntry()->getType());
            vec1.push_back(temp->getId()->getSymbolEntry());
            temp = dynamic_cast<DeclStmt*>(temp->getNext());
        }
        funcType = new FunctionType($1, vec, vec1);
        SymbolEntry* se = new IdentifierSymbolEntry(
            funcType, $2, identifiers->getPrev()->getLevel());
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setIntParamNo(IntParamNum);
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setFloatParamNo(FloatParamNum);
        if (!identifiers->getPrev()->install($2, se)) {
            printTyCk("redefinition of " << $2 << " " << se->getType()->toStr().c_str());
        }
        $<se>$ = se; 
    } 
    BlockState {
        $$ = new FunctionDef($<se>7, dynamic_cast<DeclStmt*>($5), $8);
        SymbolTable* top = identifiers;
        identifiers = identifiers->getPrev();
        delete top;
        delete []$2;
    }
    ;
FuncRParams 
    : Cond {$$ = $1;}
    | FuncRParams COMMA Cond {
        $$ = $1;
        $$->setNext($3);
    }
FuncFParamsList
    : FuncFParams {$$ = $1;}
    | %empty {$$ = nullptr;}
FuncFParams
    : FuncFParams COMMA FuncFParam {
        $$ = $1;
        $$->setNext($3);
    }
    | FuncFParam {
        $$ = $1;
    }
    ;
FuncFParam
    : Type ID {
        SymbolEntry* se;
        if ($1->isFloat()) {
            *fcheck=1;
            se = new IdentifierSymbolEntry($1, $2, identifiers->getLevel(), FloatParamNum++);
            (dynamic_cast<IdentifierSymbolEntry*>(se))->setAllParamNo(FloatParamNum + IntParamNum - 1);
            if (FloatParamNum > 4){
                (dynamic_cast<IdentifierSymbolEntry*>(se))->setStackParamNo(AllParamNum);
                AllParamNum++;
            }
        } else {
            se = new IdentifierSymbolEntry($1, $2, identifiers->getLevel(), IntParamNum++);
            (dynamic_cast<IdentifierSymbolEntry*>(se))->setAllParamNo(FloatParamNum + IntParamNum - 1);
            if (IntParamNum > 4){
                (dynamic_cast<IdentifierSymbolEntry*>(se))->setStackParamNo(AllParamNum);
                AllParamNum++;
            }
        }
        identifiers->install($2, se);
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setLabel();
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setAddr(new Operand(se));
        $$ = new DeclStmt(new Id(se));
        delete []$2;
    }
    | Type ID ArrayIndices {
        if ($1->isFloat()) *fcheck=1;
            
        ExprNode* temp = $3;
        Type* arr = $1;
        Type* arr1;
        std::vector<ExprNode*> InitValuesVec;
        while (temp) {
            InitValuesVec.push_back(temp);
            temp = dynamic_cast<ExprNode*>(temp->getNext());
        }
        while (!InitValuesVec.empty()) {
            arr1 = new ArrayType(arr, InitValuesVec.back()->getValue());
            if (arr->isArray())
                (dynamic_cast<ArrayType*>(arr))->setArrayType(arr1);
            arr = arr1;
            InitValuesVec.pop_back();
        }
        SymbolEntry* se = new IdentifierSymbolEntry(arr, $2, identifiers->getLevel(), IntParamNum++);
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setAllParamNo(FloatParamNum + IntParamNum - 1);
        if (IntParamNum > 4){
            (dynamic_cast<IdentifierSymbolEntry*>(se))->setStackParamNo(AllParamNum);
            AllParamNum++;
        }
        identifiers->install($2, se);
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setLabel();
        (dynamic_cast<IdentifierSymbolEntry*>(se))->setAddr(new Operand(se));
        $$ = new DeclStmt(new Id(se));
        delete []$2;
    }
    ;
%%

int yyerror(char const* message)
{
    std::cerr<<message<<std::endl;
    return -1;
}
